@@ -127,7 +127,7 @@ static char *cluster_set_resource_string(struct batch_queue *q, const struct rms
 		free(mem);
 	}
 	const char *safe_mode = hash_table_lookup(q->options, "safe-submit-mode");
-	if(!strcmp("yes", safe_mode))
+	if(safe_mode && !strcmp("yes", safe_mode))
 		free(cluster_resources);
 	if(!cluster_resources)
 		cluster_resources = xxstrdup("");
