@@ -1429,7 +1429,7 @@ static int handle_master(struct link *master) {
 			if(pattern_match(line, "^put (.+) (%d+) ([0-7]+) (%d+)$", &f, &l, &m, &g) >= 0) {
 				strncpy(filename, f, WORK_QUEUE_LINE_MAX); free(f);
 				length = strtoll(l, 0, 10); free(l);
-				mode   = atoi(m);           free(m);
+				mode   = strtol(m, NULL, 8); free(m);
 				free(g); //flags are not used anymore.
 				
 				if(path_within_dir(filename, workspace)) {
@@ -1676,10 +1676,12 @@ static void work_for_master(struct link *master) {
 		 * Mark offending process as RESOURCE_EXHASTION. */
 		enforce_processes_limits();
 
-		/* end running processes if worker resources are exhasusted, and
-		 * marked them as RESOURCE_EXHAUSTION. */
+		/* end running processes if worker resources are exhasusted, and marked
+		 * them as FORSAKEN, so they can be resubmitted somewhere else. */
 		if(!enforce_worker_limits(master)) {
-			finish_running_tasks(WORK_QUEUE_RESULT_RESOURCE_EXHAUSTION);
+			finish_running_tasks(WORK_QUEUE_RESULT_FORSAKEN);
+			// finish all tasks, disconnect from master, but don't kill the worker (no abort_flag = 1)
+			break;
 		}
 
 		int task_event = 0;
@@ -1716,7 +1718,9 @@ static void work_for_master(struct link *master) {
 		}
 
 
-		if(!ok) break;
+		if(!ok) {
+			break;
+		}
 
 		//Reset idle_stoptime if something interesting is happening at this worker.
 		if(list_size(procs_waiting) > 0 || itable_size(procs_table) > 0 || itable_size(procs_complete) > 0) {
